import os
from pulsar.apps import wsgi

__all__ = ['WsgiMeta', 'PulsarApp']


class WsgiMeta(type):
    def __call__(cls, *args, **kwargs):
        return super(WsgiMeta, cls).__call__(*args, **kwargs)


class PulsarApp(wsgi.LazyWsgi, metaclass=WsgiMeta):
    def __init__(self, app=None, middleware=None):
        self.application = app
        self.middleware = middleware
        return super(PulsarApp, self).__init__()

    def getapp(self):
        module = __import__(self.application)
        return getattr(module, 'application', getattr(module, 'app', module))

    def getmiddlemare(self):
        if not self.middleware:
            return []
        module = __import__(self.middleware)
        return getattr(module, 'middleware', getattr(module, 'mdw', module))

    def setup(self, environ=os.environ):
        app = self.getapp()
        middlewares = self.getmiddlemare() + [
            wsgi.wait_for_body_middleware,
            wsgi.middleware_in_executor(app)]
        return wsgi.WsgiHandler(tuple(middlewares))



__all__ = ['router']


def router(app, rule, method):
    '''Map a function to :class:`Router` and add to the :attr:`routes` list.
    Typical usage:

    app = WsgiApplication('/')

    @app.router('/hello', method='post')
    def world(request):
        return wsgi.WsgiResponse(200, 'world')
    '''
    def handler(fn):
        app.add_child(app.make_router(rule, method, fn))
        return fn
    return handler



from argparse import ArgumentParser
from pwsgi import __version__
import os
import sys
from pulsar.apps import wsgi
from pwsgi.wrapper import PulsarApp

parser = ArgumentParser(add_help=False, description='pWsgi - a pulsar based async wsgi implentation %s' % __version__)
parser.usage = 'pwsgi [-h]'
parser.add_argument('-a', '--app', help='path of wsgi app, required')
parser.add_argument('-p', '--work_path', help='work path of wsgi app [default: ./wsgiapp.py]', default='./')
parser.add_argument('-m', '--middleware', help='middleware')


args, tails = parser.parse_known_args()
os.chdir(args.work_path)
sys.path.append('.')


def load_wsgi(application, middleware=None):
    return PulsarApp(application, middleware=middleware)


def main(**kwargs):
    if len(sys.argv) == 1 or '-h' in tails:
        print(parser.format_help())
        print('-----------------------------')
    sys.argv = [sys.argv[0]] + tails
    if args.app:
        middleware = args.middleware
        pwsgi_app = load_wsgi(args.app, middleware=middleware)
        return wsgi.WSGIServer(pwsgi_app).start()


if __name__ == '__main__':
    try:
        main()
    except InterruptedError:
        exit(1)



# coding:utf8

import os
import codecs
import re
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
_CWD = os.path.dirname(__file__)

NAME = 'pwsgi'
DESCRIPTION = 'pWSGI is a async wsgi implentation based on pulsar wsgi'
AUTHOR = 'ryankung'
EMAIL = 'ryankung@ieee.org'
INSTALL_REQUIRES = [i for i in open(os.path.join(_CWD, 'requirements.txt')).readlines()
                    if not i.startswith(('-', '#', '\n'))]


def read(*parts):
    # intentionally *not* adding an encoding option to open, See:
    #   https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    return codecs.open(os.path.join(here, *parts), 'r').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name=NAME,
    description=DESCRIPTION,
    long_description=open(os.path.join(here, 'README.md')).read(),
    version=find_version('pwsgi', '__init__.py'),
    packages=find_packages(exclude=['examples', 'tests', 'docs']),
    install_requires=INSTALL_REQUIRES,
    package_dir={'': '.'},
    author=AUTHOR,
    author_email=EMAIL,
    license="MIT",
    platforms=['any'],
    url="",
    classifiers=["Intended Audience :: Developers",
                 "Programming Language :: Python",
                 "Topic :: Software Development :: Libraries :: Python Modules"],
    test_suite='nose.collector',
    tests_require=['nose'],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'pwsgi=pwsgi.main:main'
        ]
    }
)
